EID_OK = 0
EID_EAT_ERROR = 2104301300
EID_EAT_INVALID_ORDER_VOLUME = 2104301301
EID_EAT_INVALID_MARKET_ORDER_OPEN_PRICE = 2104301302
EID_EAT_INVALID_LIMIT_ORDER_OPEN_PRICE = 2104301303
EID_EAT_INVALID_STOP_ORDER_OPEN_PRICE = 2104301304
EID_EAT_INVALID_ORDER_TYPE = 2104301305
EID_EAT_INVALID_ORDER_CLOSE_PRICE = 2104301306
EID_EAT_INVALID_ORDER_STOP_LOSS = 2104301307
EID_EAT_INVALID_ORDER_TAKE_PROFIT = 2104301308
EID_EAT_NOT_ENOUGH_MONEY = 2104301309
EID_EAT_INVALID_ORDER_TICKET = 2104301310
EID_EAT_TEST_STOP = 2104301311
EID_EAT_EA_DEAD = 2104301312
EID_EAT_LIMIT_ORDERS = 2104301313
EID_EAT_INVALID_SYMBOL = 2104301314
EID_EAT_INVALID_DATA_FORMAT = 21043015
EID_EAT_INVALID_DATA_LENGTH = 21043016


class PXErrorCode(Exception):
    ''''''
    def __init__(self, errid):
        self.errid = errid

